@@ -1,13 +1,16 @@
 /***************************************************************************
  *
- * $Header: /home/werner/ytree/RCS/config.h,v 1.17 1998/11/11 19:02:48 werner Rel $
+ * $Header: /usr/local/cvsroot/utils/ytree/config.h,v 1.19 2000/07/13 18:26:06 werner Exp $
  *
  * Header-Datei fuer YTREE
  *
  ***************************************************************************/
 
 /* Default (max) TreeDepth on startup */
-#define DEFAULT_TREEDEPTH  "1"
+#define DEFAULT_TREEDEPTH     "1"
+#define DEFAULT_FILEMODE      "2"
+#define DEFAULT_NOSMALLWINDOW "0"
+#define DEFAULT_NUMBERSEP     "."
 
 /* UNIX-Kommandos */
 /*----------------*/
@@ -32,6 +35,9 @@
 #define DEFAULT_ARCLIST    "arc v"
 #define DEFAULT_ARCEXPAND  "arc p"
 
+#define DEFAULT_RARLIST    "unrar l"
+#define DEFAULT_RAREXPAND  "unrar p -c- -INUL"
+
 #define DEFAULT_TAPEDEV    "/dev/rmt0"
 
 
