@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /home/werner/ytree/RCS/copy.c,v 1.11 1997/12/27 16:07:35 werner Rel $
+ * $Header: /usr/local/cvsroot/utils/ytree/copy.c,v 1.14 2000/06/10 13:15:41 werner Exp $
  *
  * Kopieren von Dateien / Verzeichnissen
  *
@@ -25,9 +25,11 @@ int CopyFile(Statistic *statistic_ptr,
              BOOL path_copy
 	    )
 {
-  unsigned    long file_size;
+  LONGLONG    file_size;
   char        from_path[PATH_LENGTH+1];
+  char        from_dir[PATH_LENGTH+1];
   char        to_path[PATH_LENGTH+1];
+  char        abs_path[PATH_LENGTH+1];
   char        buffer[20];
   FileEntry   *dest_file_entry;
   FileEntry   *fen_ptr;
@@ -39,6 +41,7 @@ int CopyFile(Statistic *statistic_ptr,
   result = -1;
 
   (void) GetRealFileNamePath( fe_ptr, from_path );
+  (void) GetPath(fe_ptr->dir_entry, from_dir);
 
   *to_path = '\0';
   if( strcmp( to_dir_path, FILE_SEPARATOR_STRING ) )
@@ -57,6 +60,13 @@ int CopyFile(Statistic *statistic_ptr,
     strcat(to_path, FILE_SEPARATOR_STRING );
     if ((tmpdir = opendir(to_path)) == NULL)
     {
+       if(*to_path != FILE_SEPARATOR_CHAR) {
+         strcpy(abs_path, from_dir);
+	 strcat(abs_path, FILE_SEPARATOR_STRING);
+	 strcat(abs_path, to_path);
+	 strcpy(to_path, abs_path);
+       }
+
        if( MakePath( statistic_ptr->tree, to_path, &dest_dir_entry ) )
        {
        	   (void) sprintf( message, 
@@ -73,9 +83,15 @@ int CopyFile(Statistic *statistic_ptr,
   }
   (void) strcat( to_path, FILE_SEPARATOR_STRING );
   if ((tmpdir = opendir(to_path)) == NULL)
-    if (errno == ENOENT)
+    if (errno == ENOENT) {
      if ( (term =InputChoise( "Directory does not exist; create (y/N) ? ", "YN\033" ))== 'Y') 
      {
+        if(*to_path != FILE_SEPARATOR_CHAR) {
+          strcpy(abs_path, from_dir);
+	  strcat(abs_path, FILE_SEPARATOR_STRING);
+	  strcat(abs_path, to_path);
+	  strcpy(to_path, abs_path);
+        }
         if (MakePath(statistic_ptr->tree, to_path, &dest_dir_entry ) )
         {
                 closedir(tmpdir);
@@ -93,6 +109,7 @@ int CopyFile(Statistic *statistic_ptr,
         closedir(tmpdir);
         return ( result );
      }
+  }
   (void) strcat( to_path, to_file );
 
 
