@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /home/werner/ytree/RCS/dirwin.c,v 1.13 1998/05/10 20:02:24 werner Rel $
+ * $Header: /usr/local/cvsroot/utils/ytree/dirwin.c,v 1.18 2000/07/13 18:26:06 werner Exp $
  *
  * Funktionen zur Handhabung des DIR-Windows
  *
@@ -10,18 +10,13 @@
 #include "ytree.h"
 
 
-#define MIN( a, b ) ( ( (a) < (b) ) ? (a) : (b) )
-#define MAX( a, b ) ( ( (a) > (b) ) ? (a) : (b) )
-
-
-
 static DirEntryList *dir_entry_list;
 static int current_dir_entry;
 static int total_dirs;
+static int window_height, window_width;
 
 static void ReadDirList(DirEntry *dir_entry);
 static void PrintDirEntry(WINDOW *win, int entry_no, int y, unsigned char hilight);
-static void DisplayTree(WINDOW *win, int start_entry_no, int hilight_no);
 static void BuildDirEntryList(DirEntry *dir_entry);
 
 static int dir_mode;
@@ -112,10 +107,10 @@ static void PrintDirEntry(WINDOW *win,
                           int y, 
                           unsigned char hilight)
 {
-  unsigned int j;
-  int n; 
-  int aux=0;
-  int color, hi_color;
+  unsigned int j, l1, l2;
+  int  n; 
+  int  aux=0;
+  int  color, hi_color;
   char buffer[32*3+PATH_LENGTH+1];
   char format[60];
   char *line_buffer = NULL;
@@ -129,6 +124,7 @@ static void PrintDirEntry(WINDOW *win,
   char *owner_name_ptr;
   char *group_name_ptr;
   DirEntry *de_ptr;
+  BOOL suppress_output = FALSE;
 
 
   if(win == f2_window) {
@@ -163,13 +159,22 @@ static void PrintDirEntry(WINDOW *win,
                     ERROR_MSG("malloc() Failed*Abort");
                     exit(1);
                  }
+#ifdef HAS_LONGLONG
+                 (void) strcpy( format, "%10s %3d %7lld %12s");
+                 
+		 (void) sprintf( line_buffer, format, attributes,
+                                 de_ptr->stat_struct.st_nlink,
+                                 (LONGLONG) de_ptr->stat_struct.st_size,
+                                 modify_time
+                                 );
+#else
                  (void) strcpy( format, "%10s %3d %7d %12s");
                  (void) sprintf( line_buffer, format, attributes,
                                  de_ptr->stat_struct.st_nlink,
                                  de_ptr->stat_struct.st_size,
                                  modify_time
                                  );
-                 aux = 36;
+#endif
 		 break;
     case MODE_2 :
                  (void)GetAttributes(de_ptr->stat_struct.st_mode, attributes);
@@ -196,10 +201,8 @@ static void PrintDirEntry(WINDOW *win,
                                  de_ptr->stat_struct.st_ino, 
                                  owner_name_ptr,         
                                  group_name_ptr);
-                 aux = 37;
                  break;
-    case MODE_3 : aux = 0;
-		  break;
+    case MODE_3 : break;
     case MODE_4 : 
                  (void) CTime( de_ptr->stat_struct.st_ctime, change_time );
                  (void) CTime( de_ptr->stat_struct.st_atime, access_time );
@@ -210,49 +213,72 @@ static void PrintDirEntry(WINDOW *win,
                     exit(1);
                  }
                  (void)sprintf(line_buffer, format, change_time, access_time);
-                 aux = 38;
                  break;
   }
   
-  WbkgdSet(win, COLOR_PAIR(color)| A_BOLD );
-  if (line_buffer)
-     mvwaddstr(win, y, 0, line_buffer );
-  n = (int) strlen( buffer );
-  n = DIR_WINDOW_WIDTH - n - aux - 1;
-  if (n > 1)
-     PrintSpecialString( win, y, aux, buffer, color );
-  else
-     n = DIR_WINDOW_WIDTH - aux ;
-/*     PrintSpecialString( win, y, aux, " ===== ", A_REVERSE );*/
-  dir_name = de_ptr->name;
-  (void) strcpy( buffer, ( *dir_name ) ? dir_name : "." );
-  if( de_ptr->not_scanned ) {
-    (void) strcat( buffer, "/" );
-  }
-  /* Scrollbares Dir-Window noch nicht implementiert... Sorry */
-  /*----------------------------------------------------------*/
-  aux = n;
-  n = strlen(buffer);
-  buffer[(n > aux) ? aux : n] = '\0';
+  if(window_width == 1)
+    suppress_output = TRUE;
 
+  aux = 0;
+  /* Output optional Attributes */
+  if (!suppress_output) {
+     WbkgdSet(win, COLOR_PAIR(color)| A_BOLD );
+     if(line_buffer) {
+       aux = strlen(line_buffer);
+       if(window_width < aux) {
+         line_buffer[window_width] = '\0';
+         suppress_output = TRUE;
+       }
+       mvwaddstr(win, y, 0, line_buffer );
+    }
+  }
+
+  if(!suppress_output) {
+    /* Output Graph */
+    l1 = strlen(buffer);
+    n = window_width - aux;
+    if(l1 > n) {
+       buffer[MAXIMUM(n, 0)] = '\0';
+       suppress_output = TRUE;
+    }
+    PrintSpecialString( win, y, aux, buffer, color );
+  }
+
+  if(!suppress_output) {
+
+    /* Output Dirname */
+
+    dir_name = de_ptr->name;
+    (void) strcpy( buffer, ( *dir_name ) ? dir_name : "." );
+    if( de_ptr->not_scanned ) {
+      (void) strcat( buffer, "/" );
+    }
+    
 #ifdef NO_HIGHLIGHT
-  (void) strcat( buffer, (hilight) ? " <" : "  " ); 
+    (void) strcat( buffer, (hilight) ? " <" : "  " ); 
 #else /* NO_HIGHLIGHT */
 #ifdef COLOR_SUPPORT
-  if( hilight ) 
-    WbkgdSet(win, COLOR_PAIR(hi_color)|A_BOLD);
-  else
-    WbkgdSet(win, COLOR_PAIR(color));
+    if( hilight ) 
+      WbkgdSet(win, COLOR_PAIR(hi_color)|A_BOLD);
+    else
+      WbkgdSet(win, COLOR_PAIR(color));
 
-  waddstr( win, buffer );
-  WbkgdSet(win, COLOR_PAIR(color)|A_BOLD);
+    n = window_width - aux - l1;
+    l2 = strlen(buffer);
+    if(l2 > n)
+      buffer[MAXIMUM(n, 0)] = '\0';
+
+    waddstr( win, buffer );
+    WbkgdSet(win, COLOR_PAIR(color)|A_BOLD);
 
 #else
-  if( hilight ) wattrset( win, A_REVERSE );
-  waddstr( win, buffer );
-  if( hilight ) wattrset( win, 0 );
+    if( hilight ) wattrset( win, A_REVERSE );
+    waddstr( win, buffer );
+    if( hilight ) wattrset( win, 0 );
 #endif /* COLOR_SUPPORT */
 #endif /* NO_HIGHLIGHT */
+  }
+
   if (line_buffer)
      free(line_buffer);
 }
@@ -260,13 +286,17 @@ static void PrintDirEntry(WINDOW *win,
 
 
 
-static void DisplayTree(WINDOW *win, int start_entry_no, int hilight_no)
+void DisplayTree(WINDOW *win, int start_entry_no, int hilight_no)
 {
   int i, y;
   int width, height;
  
   y = -1;
   GetMaxYX(win, &height, &width);
+  if(win == dir_window) {
+    window_width  = width;
+    window_height = height;
+  }
   for(i=0; i < height; i++)
   {
     wmove(win, i, 0);
@@ -299,7 +329,7 @@ static void Movedown(int *disp_begin_pos, int *cursor_pos, DirEntry **dir_entry)
    {
       /* Element vorhanden */
       /*-------------------*/
-      if( *cursor_pos + 1 < DIR_WINDOW_HEIGHT )
+      if( *cursor_pos + 1 < window_height )
       {
           /* Element ist ohne scrollen erreichbar */
           /*--------------------------------------*/
@@ -333,7 +363,9 @@ static void Movedown(int *disp_begin_pos, int *cursor_pos, DirEntry **dir_entry)
                          );
       }
       *dir_entry = dir_entry_list[*disp_begin_pos + *cursor_pos].dir_entry;
-      DisplayFileWindow( *dir_entry, 0, -1 );
+      (*dir_entry)->start_file = 0;
+      (*dir_entry)->cursor_pos = -1;
+      DisplayFileWindow( *dir_entry );
       wnoutrefresh( file_window );
    }
    return;
@@ -387,7 +419,9 @@ static void Moveup(int *disp_begin_pos, int *cursor_pos, DirEntry **dir_entry)
                          );
       }
       *dir_entry = dir_entry_list[*disp_begin_pos + *cursor_pos].dir_entry;
-      DisplayFileWindow( *dir_entry, 0, -1 );
+      (*dir_entry)->start_file = 0;
+      (*dir_entry)->cursor_pos = -1;
+      DisplayFileWindow( *dir_entry );
       wnoutrefresh( file_window );
    }
    return;
@@ -404,7 +438,7 @@ static void Movenpage(int *disp_begin_pos, int *cursor_pos, DirEntry **dir_entry
    }
    else
    {
-      if( *cursor_pos < DIR_WINDOW_HEIGHT - 1 )
+      if( *cursor_pos < window_height - 1 )
       {
           /* Cursor steht nicht auf letztem Eintrag
            * der Seite
@@ -414,12 +448,14 @@ static void Movenpage(int *disp_begin_pos, int *cursor_pos, DirEntry **dir_entry
                           *cursor_pos,
                           FALSE
                           );
-           if( *disp_begin_pos + DIR_WINDOW_HEIGHT > total_dirs  - 1 )
+           if( *disp_begin_pos + window_height > total_dirs  - 1 )
               *cursor_pos = total_dirs - *disp_begin_pos - 1;
            else
-              *cursor_pos = DIR_WINDOW_HEIGHT - 1;
+              *cursor_pos = window_height - 1;
            *dir_entry = dir_entry_list[*disp_begin_pos + *cursor_pos].dir_entry;
-           DisplayFileWindow( *dir_entry, 0, -1 );
+      	   (*dir_entry)->start_file = 0;
+      	   (*dir_entry)->cursor_pos = -1;
+           DisplayFileWindow( *dir_entry );
            wnoutrefresh( file_window );
            PrintDirEntry( dir_window,
                           *disp_begin_pos + *cursor_pos, 
@@ -431,19 +467,21 @@ static void Movenpage(int *disp_begin_pos, int *cursor_pos, DirEntry **dir_entry
       {
           /* Scrollen */
           /*----------*/
-          if( *disp_begin_pos + *cursor_pos + DIR_WINDOW_HEIGHT < total_dirs )
+          if( *disp_begin_pos + *cursor_pos + window_height < total_dirs )
           {
-              *disp_begin_pos += DIR_WINDOW_HEIGHT;
-              *cursor_pos = DIR_WINDOW_HEIGHT - 1;
+              *disp_begin_pos += window_height;
+              *cursor_pos = window_height - 1;
           }
           else
           {
-              *disp_begin_pos = total_dirs - DIR_WINDOW_HEIGHT;
+              *disp_begin_pos = total_dirs - window_height;
               if( *disp_begin_pos < 0 ) *disp_begin_pos = 0;
               *cursor_pos = total_dirs - *disp_begin_pos - 1;
           }
           *dir_entry = dir_entry_list[*disp_begin_pos + *cursor_pos].dir_entry;
-          DisplayFileWindow( *dir_entry, 0, -1 );
+      	  (*dir_entry)->start_file = 0;
+      	  (*dir_entry)->cursor_pos = -1;
+          DisplayFileWindow( *dir_entry );
           wnoutrefresh( file_window );
           DisplayTree(dir_window,*disp_begin_pos,*disp_begin_pos+*cursor_pos);
       }
@@ -473,7 +511,9 @@ static void Moveppage(int *disp_begin_pos, int *cursor_pos, DirEntry **dir_entry
                           );
            *cursor_pos = 0;
            *dir_entry = dir_entry_list[*disp_begin_pos + *cursor_pos].dir_entry;
-           DisplayFileWindow( *dir_entry, 0, -1 );
+      	   (*dir_entry)->start_file = 0;
+      	   (*dir_entry)->cursor_pos = -1;
+           DisplayFileWindow( *dir_entry );
            wnoutrefresh( file_window );
            PrintDirEntry( dir_window,
                           *disp_begin_pos + *cursor_pos, 
@@ -485,13 +525,15 @@ static void Moveppage(int *disp_begin_pos, int *cursor_pos, DirEntry **dir_entry
       {
          /* Scrollen */
          /*----------*/
-         if( (*disp_begin_pos -= DIR_WINDOW_HEIGHT) < 0 )
+         if( (*disp_begin_pos -= window_height) < 0 )
          {
              *disp_begin_pos = 0;
          }
          *cursor_pos = 0;
          *dir_entry = dir_entry_list[*disp_begin_pos + *cursor_pos].dir_entry;
-         DisplayFileWindow( *dir_entry, 0, -1 );
+      	 (*dir_entry)->start_file = 0;
+      	 (*dir_entry)->cursor_pos = -1;
+         DisplayFileWindow( *dir_entry );
          wnoutrefresh( file_window );
          DisplayTree(dir_window,*disp_begin_pos,*disp_begin_pos+*cursor_pos);
       } 
@@ -500,10 +542,7 @@ static void Moveppage(int *disp_begin_pos, int *cursor_pos, DirEntry **dir_entry
 }
 
 
-int HandleDirWindow(DirEntry *start_dir_entry,
-		    int disp_begin_pos,
-		    int cursor_pos
-		   )
+int HandleDirWindow(DirEntry *start_dir_entry)
 {
   DirEntry  *dir_entry, *de_ptr;
   FileEntry *fe_ptr;
@@ -514,30 +553,32 @@ int HandleDirWindow(DirEntry *start_dir_entry,
 
   unput_char = 0;
 
+  GetMaxYX(dir_window, &window_height, &window_width);
 
   /* Merker loeschen */
   /*-----------------*/
 
-  statistic.disp_begin_pos = 0;
-  statistic.cursor_pos     = 0;
   dir_mode = MODE_3;
 
   need_dsp_help = TRUE;
 
   BuildDirEntryList( start_dir_entry );
   
-  dir_entry = dir_entry_list[disp_begin_pos + cursor_pos].dir_entry;
+  dir_entry = dir_entry_list[statistic.disp_begin_pos + statistic.cursor_pos].dir_entry;
  
   DisplayDiskStatistic();
 
-  DisplayFileWindow( dir_entry, 0, -1 );
+  if(!dir_entry->login_flag) {
+    dir_entry->start_file = 0;
+    dir_entry->cursor_pos = -1;
+  }
+  DisplayFileWindow( dir_entry );
   wnoutrefresh( file_window );
-  DisplayTree( dir_window, disp_begin_pos, disp_begin_pos + cursor_pos );
+  DisplayTree( dir_window, statistic.disp_begin_pos, statistic.disp_begin_pos + statistic.cursor_pos );
   touchwin(dir_window);
 
   if( dir_entry->login_flag )
   {
-    dir_entry->login_flag = FALSE;
     if( dir_entry->global_flag )
     {
       unput_char = 'S';
@@ -564,24 +605,55 @@ int HandleDirWindow(DirEntry *start_dir_entry,
     else
     {
       doupdate();
-      ch = getch();
+      ch = (resize_request) ? -1 : Getch();
       if( ch == LF ) ch = CR;
     }
 #ifdef VI_KEYS
     ch = ViKey( ch );
 #endif /* VI_KEYS */
+
+
+    if(resize_request) {
+       ReCreateWindows();
+       DisplayMenu();
+       GetMaxYX(dir_window, &window_height, &window_width);
+       while(statistic.cursor_pos >= window_height) {
+         statistic.cursor_pos--;
+	 statistic.disp_begin_pos++;
+       }
+       DisplayTree( dir_window, statistic.disp_begin_pos, 
+		    statistic.disp_begin_pos + statistic.cursor_pos 
+		  );
+       DisplayFileWindow(dir_entry);
+       DisplayDiskStatistic();
+       DisplayDirParameter( dir_entry );
+       need_dsp_help = TRUE;
+       DisplayAvailBytes();
+       DisplayFileSpec();
+       DisplayDiskName();
+       resize_request = FALSE;
+    }
+
     switch( ch )
     {
+
+#ifdef KEY_RESIZE
+      case KEY_RESIZE: resize_request = TRUE;
+      		       break;
+#endif
+
+      case -1:        break;
+
       case ' ':      /*break;   Quick-Key */
-      case KEY_DOWN: Movedown(&disp_begin_pos, &cursor_pos, &dir_entry);
+      case KEY_DOWN: Movedown(&statistic.disp_begin_pos, &statistic.cursor_pos, &dir_entry);
                      break;
-      case KEY_UP  : Moveup(&disp_begin_pos, &cursor_pos, &dir_entry);
+      case KEY_UP  : Moveup(&statistic.disp_begin_pos, &statistic.cursor_pos, &dir_entry);
                      break;
-      case KEY_NPAGE:Movenpage(&disp_begin_pos, &cursor_pos, &dir_entry);		     
+      case KEY_NPAGE:Movenpage(&statistic.disp_begin_pos, &statistic.cursor_pos, &dir_entry);		     
                      break;
-      case KEY_PPAGE:Moveppage(&disp_begin_pos, &cursor_pos, &dir_entry);
+      case KEY_PPAGE:Moveppage(&statistic.disp_begin_pos, &statistic.cursor_pos, &dir_entry);
                      break;
-      case KEY_HOME: if( disp_begin_pos == 0 && cursor_pos == 0 )
+      case KEY_HOME: if( statistic.disp_begin_pos == 0 && statistic.cursor_pos == 0 )
 		     { 
 		       /* Position 1 bereits errecht */
 		       /*----------------------------*/
@@ -589,23 +661,27 @@ int HandleDirWindow(DirEntry *start_dir_entry,
 		     }
 		     else
 		     {
-		       disp_begin_pos = 0;
-		       cursor_pos     = 0;
-		       dir_entry = dir_entry_list[disp_begin_pos + cursor_pos].dir_entry;
-                       DisplayFileWindow( dir_entry, 0, -1 );
+		       statistic.disp_begin_pos = 0;
+		       statistic.cursor_pos     = 0;
+		       dir_entry = dir_entry_list[statistic.disp_begin_pos + statistic.cursor_pos].dir_entry;
+		       dir_entry->start_file = 0;
+		       dir_entry->cursor_pos = -1;
+                       DisplayFileWindow( dir_entry );
                        wnoutrefresh( file_window );
-                       DisplayTree( dir_window, disp_begin_pos, 
-				    disp_begin_pos + cursor_pos 
+                       DisplayTree( dir_window, statistic.disp_begin_pos, 
+				    statistic.disp_begin_pos + statistic.cursor_pos 
 				  );
 		     }
                      break;
-      case KEY_END : disp_begin_pos = MAX(0, total_dirs - DIR_WINDOW_HEIGHT);
-		     cursor_pos     = total_dirs - disp_begin_pos - 1;
-		     dir_entry = dir_entry_list[disp_begin_pos + cursor_pos].dir_entry;
-                     DisplayFileWindow( dir_entry, 0, -1 );
+      case KEY_END : statistic.disp_begin_pos = MAXIMUM(0, total_dirs - window_height);
+		     statistic.cursor_pos     = total_dirs - statistic.disp_begin_pos - 1;
+		     dir_entry = dir_entry_list[statistic.disp_begin_pos + statistic.cursor_pos].dir_entry;
+		     dir_entry->start_file = 0;
+		     dir_entry->cursor_pos = -1;
+                     DisplayFileWindow( dir_entry );
                      wnoutrefresh( file_window );
-                     DisplayTree( dir_window, disp_begin_pos, 
-				  disp_begin_pos + cursor_pos 
+                     DisplayTree( dir_window, statistic.disp_begin_pos, 
+				  statistic.disp_begin_pos + statistic.cursor_pos 
 				);
                      break;
       case KEY_RIGHT:
@@ -619,8 +695,8 @@ int HandleDirWindow(DirEntry *start_dir_entry,
 		        }
 		        dir_entry->not_scanned = FALSE;
 	                BuildDirEntryList( start_dir_entry );
-                        DisplayTree( dir_window, disp_begin_pos, 
-		                     disp_begin_pos + cursor_pos 
+                        DisplayTree( dir_window, statistic.disp_begin_pos, 
+		                     statistic.disp_begin_pos + statistic.cursor_pos 
 		                   );
 			DisplayDiskStatistic();
 	                DisplayAvailBytes();
@@ -630,8 +706,8 @@ int HandleDirWindow(DirEntry *start_dir_entry,
       case '\t':
       case '*':       ScanSubTree( dir_entry );
 	              BuildDirEntryList( start_dir_entry );
-                      DisplayTree( dir_window, disp_begin_pos, 
-		                   disp_begin_pos + cursor_pos 
+                      DisplayTree( dir_window, statistic.disp_begin_pos, 
+		                   statistic.disp_begin_pos + statistic.cursor_pos 
 		                 );
 		      DisplayDiskStatistic();
 	              DisplayAvailBytes();
@@ -648,18 +724,21 @@ int HandleDirWindow(DirEntry *start_dir_entry,
 			}
 		        dir_entry->not_scanned = TRUE;
 	                BuildDirEntryList( start_dir_entry );
-                        DisplayTree( dir_window, disp_begin_pos, 
-		                     disp_begin_pos + cursor_pos 
+                        DisplayTree( dir_window, statistic.disp_begin_pos, 
+		                     statistic.disp_begin_pos + statistic.cursor_pos 
 		                   );
 	                DisplayAvailBytes();
 	                need_dsp_help = TRUE;
 		      }
 	              break;
       case 'F':
-      case 'f':      ReadFileSpec();
-                     DisplayFileWindow( dir_entry, 0, -1 );
-                     wnoutrefresh( file_window );
-		     DisplayDiskStatistic();
+      case 'f':      if(ReadFileSpec() == 0) {
+		       dir_entry->start_file = 0;
+		       dir_entry->cursor_pos = -1;
+                       DisplayFileWindow( dir_entry );
+                       wnoutrefresh( file_window );
+		       DisplayDiskStatistic();
+		     }
 		     need_dsp_help = TRUE;
                      break;
       case 'T' :
@@ -677,7 +756,9 @@ int HandleDirWindow(DirEntry *start_dir_entry,
 			 }
 		       }
 		     }
-                     DisplayFileWindow( dir_entry, 0, -1 );
+		     dir_entry->start_file = 0;
+		     dir_entry->cursor_pos = -1;
+                     DisplayFileWindow( dir_entry );
                      wnoutrefresh( file_window );
 		     DisplayDiskStatistic();
 		     break;
@@ -697,7 +778,9 @@ int HandleDirWindow(DirEntry *start_dir_entry,
 			 }
 		       }
 		     }
-                     DisplayFileWindow( dir_entry, 0, -1 );
+		     dir_entry->start_file = 0;
+		     dir_entry->cursor_pos = -1;
+                     DisplayFileWindow( dir_entry );
                      wnoutrefresh( file_window );
 		     DisplayDiskStatistic();
 		     break;
@@ -722,7 +805,9 @@ int HandleDirWindow(DirEntry *start_dir_entry,
 			 }
 		       }
 		     }
-                     DisplayFileWindow( dir_entry, 0, -1 );
+		     dir_entry->start_file = 0;
+		     dir_entry->cursor_pos = -1;
+                     DisplayFileWindow( dir_entry );
                      wnoutrefresh( file_window );
 		     DisplayDiskStatistic();
 		     break;
@@ -747,15 +832,17 @@ int HandleDirWindow(DirEntry *start_dir_entry,
 			 }
 		       }
 		     }
-                     DisplayFileWindow( dir_entry, 0, -1 );
+		     dir_entry->start_file = 0;
+		     dir_entry->cursor_pos = -1;
+                     DisplayFileWindow( dir_entry );
                      wnoutrefresh( file_window );
 		     DisplayDiskStatistic();
 		     break;
       case 'F' & 0x1F : 
 		     RotateDirMode();
                      /*DisplayFileWindow( dir_entry, 0, -1 );*/
-                     DisplayTree( dir_window, disp_begin_pos, 
-                                  disp_begin_pos + cursor_pos 
+                     DisplayTree( dir_window, statistic.disp_begin_pos, 
+                                  statistic.disp_begin_pos + statistic.cursor_pos 
                                   );
                      /*wnoutrefresh( file_window );*/
 		     DisplayDiskStatistic();
@@ -764,41 +851,40 @@ int HandleDirWindow(DirEntry *start_dir_entry,
       case 'S':
       case 's':      if( statistic.disk_matching_files )
 		     {
-		       /* Cursor-Position sichern */
-		       /*-------------------------*/
-		       statistic.disp_begin_pos = disp_begin_pos;
-		       statistic.cursor_pos     = cursor_pos;
-		       if( HandleFileWindow( dir_entry, 
-					     dir_entry->start_file,
-					     dir_entry->cursor_pos,
-					     TRUE, TRUE ) != LOGIN_ESC )
+    		       if(dir_entry->login_flag) {
+		         dir_entry->login_flag = FALSE;
+		       } else {
+		         dir_entry->big_window  = TRUE;
+		         dir_entry->global_flag = TRUE;
+		         dir_entry->start_file  = 0;
+		         dir_entry->cursor_pos  = 0;
+		       }
+
+		       if( HandleFileWindow(dir_entry) != LOGIN_ESC )
 		       {
 		         DisplayDiskStatistic();
-                         DisplayFileWindow( dir_entry, 0, -1 );
+		         dir_entry->start_file = 0;
+		         dir_entry->cursor_pos = -1;
+                         DisplayFileWindow( dir_entry );
                          wnoutrefresh( small_file_window );
                          wnoutrefresh( big_file_window );
 		         BuildDirEntryList( start_dir_entry );
-                         DisplayTree( dir_window, disp_begin_pos, 
-				      disp_begin_pos + cursor_pos 
+                         DisplayTree( dir_window, statistic.disp_begin_pos, 
+				      statistic.disp_begin_pos + statistic.cursor_pos 
 				    );
-			 /* Cursor-Position zuruecksetzen */
-		         /*-------------------------------*/
-		         statistic.disp_begin_pos = 0;
-		         statistic.cursor_pos     = 0;
 		       }
 		       else
 		       {
-			 /* wie bei Login */
-			 /*---------------*/
 		         BuildDirEntryList( statistic.tree );
-                         DisplayTree( dir_window, disp_begin_pos, 
-				      disp_begin_pos + cursor_pos 
+                         DisplayTree( dir_window, statistic.disp_begin_pos, 
+				      statistic.disp_begin_pos + statistic.cursor_pos 
 				    );
 			 ch = 'L';
 		       }
 		     }
 		     else
 		     {
+		       dir_entry->login_flag = FALSE;
 		       beep();
 		     }
 		     need_dsp_help = TRUE;
@@ -806,37 +892,33 @@ int HandleDirWindow(DirEntry *start_dir_entry,
       case LF :
       case CR :      if( dir_entry->matching_files )
 		     {
-		       /* Cursor-Position sichern */
-		       /*-------------------------*/
-		       statistic.disp_begin_pos = disp_begin_pos;
-		       statistic.cursor_pos     = cursor_pos;
-		       if( HandleFileWindow( dir_entry, 
-					     dir_entry->start_file,
-					     dir_entry->cursor_pos,
-					     dir_entry->global_flag,
-					     dir_entry->big_window 
-					   ) != LOGIN_ESC )
+    		       if(dir_entry->login_flag) {
+		         dir_entry->login_flag = FALSE;
+		       } else {
+		         dir_entry->global_flag = FALSE;
+		         dir_entry->big_window  = bypass_small_window;
+		         dir_entry->start_file  = 0;
+		         dir_entry->cursor_pos  = 0;
+		       }
+
+		       if( HandleFileWindow( dir_entry ) != LOGIN_ESC )
                        {
-			 DisplayFileWindow( dir_entry, 0, -1 );
+		         dir_entry->start_file = 0;
+		         dir_entry->cursor_pos = -1;
+			 DisplayFileWindow( dir_entry );
                          wnoutrefresh( small_file_window );
                          wnoutrefresh( big_file_window );
 		         BuildDirEntryList( start_dir_entry );
-                         DisplayTree( dir_window, disp_begin_pos, 
-				      disp_begin_pos + cursor_pos 
+                         DisplayTree( dir_window, statistic.disp_begin_pos, 
+				      statistic.disp_begin_pos + statistic.cursor_pos 
 				    );
 			 DisplayDiskStatistic();
-			 /* Cursor-Position zuruecksetzen */
-		         /*-------------------------------*/
-		         statistic.disp_begin_pos = 0;
-		         statistic.cursor_pos     = 0;
 		       }
 		       else
 		       {
-			 /* wie bei Login */
-			 /*---------------*/
 		         BuildDirEntryList( statistic.tree );
-                         DisplayTree( dir_window, disp_begin_pos, 
-				      disp_begin_pos + cursor_pos 
+                         DisplayTree( dir_window, statistic.disp_begin_pos, 
+				      statistic.disp_begin_pos + statistic.cursor_pos 
 				    );
 			 ch = 'L';
 		       }
@@ -845,6 +927,7 @@ int HandleDirWindow(DirEntry *start_dir_entry,
 		     }
 		     else
 		     {
+		       dir_entry->login_flag = FALSE;
 		       beep();
 		     }
 		     break;
@@ -857,8 +940,8 @@ int HandleDirWindow(DirEntry *start_dir_entry,
       case 'm':      if( !MakeDirectory( dir_entry ) )
 		     {
 		       BuildDirEntryList( start_dir_entry );
-                       DisplayTree( dir_window, disp_begin_pos, 
-				    disp_begin_pos + cursor_pos 
+                       DisplayTree( dir_window, statistic.disp_begin_pos, 
+				    statistic.disp_begin_pos + statistic.cursor_pos 
 				  );
 		       DisplayAvailBytes();
 		     } 
@@ -866,19 +949,21 @@ int HandleDirWindow(DirEntry *start_dir_entry,
 		     break;
       case 'D':
       case 'd':      if( !DeleteDirectory( dir_entry ) ) {
-		       if( disp_begin_pos + cursor_pos > 0 )
+		       if( statistic.disp_begin_pos + statistic.cursor_pos > 0 )
 		       {
-		         if( cursor_pos > 0 ) cursor_pos--;
-		         else disp_begin_pos--;
+		         if( statistic.cursor_pos > 0 ) statistic.cursor_pos--;
+		         else statistic.disp_begin_pos--;
 		       }
       		     }
 		     /* Unabhaengig vom Erfolg aktualisieren */
 		     BuildDirEntryList( start_dir_entry );
-		     dir_entry = dir_entry_list[disp_begin_pos + cursor_pos].dir_entry;
-                     DisplayFileWindow( dir_entry, 0, -1 );
+		     dir_entry = dir_entry_list[statistic.disp_begin_pos + statistic.cursor_pos].dir_entry;
+		     dir_entry->start_file = 0;
+		     dir_entry->cursor_pos = -1;
+                     DisplayFileWindow( dir_entry );
                      wnoutrefresh( file_window );
-		     DisplayTree( dir_window, disp_begin_pos, 
-				  disp_begin_pos + cursor_pos 
+		     DisplayTree( dir_window, statistic.disp_begin_pos, 
+				  statistic.disp_begin_pos + statistic.cursor_pos 
 				);
 		     DisplayAvailBytes();
 		     need_dsp_help = TRUE;
@@ -891,28 +976,28 @@ int HandleDirWindow(DirEntry *start_dir_entry,
 			 /* Rename OK */
 			 /*-----------*/
 		         BuildDirEntryList( start_dir_entry );
-                         DisplayTree( dir_window, disp_begin_pos, 
-				      disp_begin_pos + cursor_pos 
+                         DisplayTree( dir_window, statistic.disp_begin_pos, 
+				      statistic.disp_begin_pos + statistic.cursor_pos 
 				    );
 		         DisplayAvailBytes();
-		         dir_entry = dir_entry_list[disp_begin_pos + cursor_pos].dir_entry;
+		         dir_entry = dir_entry_list[statistic.disp_begin_pos + statistic.cursor_pos].dir_entry;
                        }
 		     }
 		     need_dsp_help = TRUE;
 		     break;
       case 'G':
       case 'g':      (void) ChangeDirGroup( dir_entry );
-                     DisplayTree( dir_window, disp_begin_pos, disp_begin_pos + cursor_pos );
+                     DisplayTree( dir_window, statistic.disp_begin_pos, statistic.disp_begin_pos + statistic.cursor_pos );
 		     need_dsp_help = TRUE;
 		     break;
       case 'O':
       case 'o':      (void) ChangeDirOwner( dir_entry );
-                     DisplayTree( dir_window, disp_begin_pos, disp_begin_pos + cursor_pos );
+                     DisplayTree( dir_window, statistic.disp_begin_pos, statistic.disp_begin_pos + statistic.cursor_pos );
 		     need_dsp_help = TRUE;
 		     break;
       case 'A':
       case 'a':      (void) ChangeDirModus( dir_entry );
-                     DisplayTree( dir_window, disp_begin_pos, disp_begin_pos + cursor_pos );
+                     DisplayTree( dir_window, statistic.disp_begin_pos, statistic.disp_begin_pos + statistic.cursor_pos );
 		     need_dsp_help = TRUE;
 		     break;
       case 'Q':
@@ -930,10 +1015,6 @@ int HandleDirWindow(DirEntry *start_dir_entry,
 		     {
 		       DisplayMenu();
 		       doupdate();
-		       /* Cursor-Position sichern */
-		       /*-------------------------*/
-		       statistic.disp_begin_pos = disp_begin_pos;
-		       statistic.cursor_pos     = cursor_pos;
 		       (void) LoginDisk( new_login_path );
 		     }
 		     need_dsp_help = TRUE;
@@ -992,7 +1073,8 @@ int KeyF2Get(DirEntry *start_dir_entry,
   {
     wnoutrefresh( f2_window );
     doupdate();
-    ch = getch();
+    ch = Getch();
+    GetMaxYX(f2_window, &win_height, &win_width);  /* Maybe changed... */
     if( ch == LF ) ch = CR;
   
 
@@ -1004,6 +1086,7 @@ int KeyF2Get(DirEntry *start_dir_entry,
 
     switch( ch )
     {
+      case -1:       break;
       case ' ':      break;  /* Quick-Key */
       case KEY_DOWN: if( disp_begin_pos + cursor_pos + 1 >= total_dirs )
 		     { 
@@ -1135,7 +1218,7 @@ int KeyF2Get(DirEntry *start_dir_entry,
 		     }
                      break;
 
-      case KEY_END : disp_begin_pos = MAX(0, total_dirs - win_height);
+      case KEY_END : disp_begin_pos = MAXIMUM(0, total_dirs - win_height);
 		     cursor_pos     = total_dirs - disp_begin_pos - 1;
                      DisplayTree( f2_window, disp_begin_pos, 
 				  disp_begin_pos + cursor_pos);
@@ -1153,7 +1236,7 @@ int KeyF2Get(DirEntry *start_dir_entry,
       default :      beep();
 		     break;
     } /* switch */
-  } while( (ch != 'q') && (ch != ESC) && (ch != 'Q') && (ch != CR) );
+  } while( (ch != 'q') && (ch != ESC) && (ch != 'Q') && (ch != CR) && (ch != -1) );
 
   UnmapF2Window();
   return( result );
