@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /home/werner/ytree/RCS/disp.c,v 1.12 1998/07/26 21:21:13 werner Rel $
+ * $Header: /usr/local/cvsroot/utils/ytree/disp.c,v 1.16 2000/05/28 17:48:40 werner Exp $
  *
  * Menue-Geruest anzeigen
  *
@@ -23,21 +23,21 @@ static char *mask[] = {
 "6--------------------7",
 "|DISK:               |",
 "| Available          |",
-"|  Bytes:            |",
+"| KBytes:            |",
 "6--------------------7",
 "|DISK Statistics     |",
-"| [Total]              |",
-"|  Files:            |",
-"|  Bytes:            |",
-"| [Matching]           |",
-"|  Files:            |",
-"|  Bytes:            |",
-"| [Tagged]             |",
-"|  Files:            |",
-"|  Bytes:            |",
-"| [Current Directory]  |",
+"|[Total]               |",
+"| Files:             |",
+"| Bytes:             |",
+"|[Matching]            |",
+"| Files:             |",
+"| Bytes:             |",
+"|[Tagged]              |",
+"| Files:             |",
+"| Bytes:             |",
+"|[Current Directory]   |",
 "|                    |",
-"|  Bytes:            |"
+"| Bytes:             |"
 };
 
 
@@ -93,6 +93,10 @@ static char *dir_help[MAX_MODES][2] =
       "RPM-DIR   (F)ilespec (L)og (S)howall (T)ag (U)ntag e(X)ecute   (^F) dirmode  (Q)uit         ",
       "COMMANDS                                                                    "
     },
+    { /* RAR_FILE_MODE */
+      "RAR-DIR   (F)ilespec (L)og (S)howall (T)ag (U)ntag e(X)ecute   (^F) dirmode  (Q)uit         ",
+      "COMMANDS                                                                    "
+    },
     { /* TAPE_MODE */
       "TAPE-DIR  (F)ilespec (L)og (S)howall (T)ag (U)ntag e(X)ecute   (^F) dirmode  (Q)uit         ",
       "COMMANDS                                                                    "
@@ -134,6 +138,10 @@ static char *file_help[MAX_MODES][2] =
       "RPM-FILE  (C)opy (F)ilespec (H)ex (P)ipe (S)ort (T)ag (U)ntag (V)iew pathcop(Y) (Q)uit         ",
       "COMMANDS  (^F)ilemode                                                        "
     },
+    { /* RAR_FILE_MODE */
+      "RAR-FILE  (C)opy (F)ilespec (H)ex (P)ipe (S)ort (T)ag (U)ntag (V)iew pathcop(Y) (Q)uit         ",
+      "COMMANDS  (^F)ilemode                                                        "
+    },
     { /* TAPE_MODE */
       "TAPE-FILE (C)opy (F)ilespec (H)ex (P)ipe (S)ort (T)ag (U)ntag (V)iew pathcop(Y) (Q)uit         ",
       "COMMANDS  (^F)ilemode                                                        "
@@ -150,7 +158,7 @@ static void DisplayVersion(void)
   
   ClearHelp();
   (void) sprintf( version, 
-		  "ytree Version %sPL%d %s (WB)", 
+		  "ytree Version %sPL%d %s (Werner Bregulla)", 
 		  VERSION, 
 		  PATCHLEVEL, 
 		  VERSIONDATE 
@@ -165,8 +173,10 @@ void DisplayDirHelp(void)
 {
   int i;
 
-  for( i=0; i < sizeof(dir_help[mode]) / sizeof(dir_help[mode][0]); i++)
+  for( i=0; i < sizeof(dir_help[mode]) / sizeof(dir_help[mode][0]); i++) {
     PrintOptions( stdscr, LINES - 2 + i, 0, dir_help[mode][i] );
+    clrtoeol();
+  }
 }
    
 
@@ -175,8 +185,10 @@ void DisplayFileHelp(void)
 {
   int i;
 
-  for( i=0; i < sizeof(file_help[mode]) / sizeof(file_help[mode][0]); i++)
+  for( i=0; i < sizeof(file_help[mode]) / sizeof(file_help[mode][0]); i++) {
     PrintOptions( stdscr, LINES - 2 + i, 0, file_help[mode][i] );
+    clrtoeol();
+  }
 }
    
 
@@ -196,7 +208,7 @@ void ClearHelp(void)
 void DisplayMenu(void)
 {
   int    y;
-  int    lines, columns;
+  int    l, c;
   
   
   PrintSpecialString( stdscr, 0, 0, "Path: ", MENU_COLOR );
@@ -218,14 +230,14 @@ void DisplayMenu(void)
   PrintLine( stdscr, 1, 0, first_line, COLS - 22);    
   PrintMenuLine( stdscr, y, 0, last_line );
 
-  lines = sizeof(logo) / sizeof(logo[0]);
-  columns = strlen( logo[0] );
+  l = sizeof(logo) / sizeof(logo[0]);
+  c = strlen( logo[0] );
 
-  for( y=0; y < lines; y++ )
+  for( y=0; y < l; y++ )
   {
     MvWAddStr( dir_window, 
-	       y + ((DIR_WINDOW_HEIGHT - lines) >> 1), 
-	       (DIR_WINDOW_WIDTH - columns) >> 1,
+	       y + ((DIR_WINDOW_HEIGHT - l) >> 1), 
+	       (DIR_WINDOW_WIDTH - c) >> 1,
 	       logo[y]
 	     );
   }
@@ -321,13 +333,14 @@ void UnmapF2Window(void)
 static void PrintMenuLine(WINDOW *win, int y, int x, char *line)
 {
   int  i;
-  int p;
+  int p, l;
   char *buffer;
   if (strchr(line,'('))
      p = 2;
   else
      p = 0;
-  if( ( buffer = (char *)malloc( COLS + 2 + p ) ) == NULL )
+  l = COLS + 2 + p;
+  if( ( buffer = (char *)malloc( l ) ) == NULL )
   {
      ERROR_MSG( "Malloc failed*ABORT" );
      exit( 1 );
@@ -337,9 +350,10 @@ static void PrintMenuLine(WINDOW *win, int y, int x, char *line)
      p = COLS - 22;
   else
      p = COLS - 24;
-  for(i=1; i < (p); i++)
+  for(i=1; i < p; i++)
       buffer[i] = line[1];
-  (void) strcpy( &buffer[i], &line[2] );
+  (void) strncpy( &buffer[i], &line[2], l - i );
+  buffer[l-1] = '\0';
   PrintOptions( stdscr, y, x , buffer );
   free( buffer );
 }
@@ -350,16 +364,19 @@ static void PrintLine(WINDOW *win, int y, int x, char *line, int len)
 {
   int  i;
   char *buffer;
-  if( ( buffer = (char *)malloc( len + 2 ) ) == NULL )
-  {
-     ERROR_MSG( "Malloc failed*ABORT" );
-     exit( 1 );
+
+  if(len > 0) {
+    if( ( buffer = (char *)malloc( len + 2 ) ) == NULL )
+    {
+       ERROR_MSG( "Malloc failed*ABORT" );
+       exit( 1 );
+    }
+    buffer[0] = line[0];
+    for(i=1; i < (len); i++)
+        buffer[i] = line[1];
+    (void) strcpy( &buffer[i], &line[2] );
+    PrintOptions( stdscr, y, x , buffer );
+    free( buffer );
   }
-  buffer[0] = line[0];
-  for(i=1; i < (len); i++)
-      buffer[i] = line[1];
-  (void) strcpy( &buffer[i], &line[2] );
-  PrintOptions( stdscr, y, x , buffer );
-  free( buffer );
 }
 
