@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /home/werner/ytree/RCS/freesp.c,v 1.13 1998/11/11 19:02:48 werner Rel $
+ * $Header: /usr/local/cvsroot/utils/ytree/freesp.c,v 1.16 2000/07/13 18:26:06 werner Exp $
  *
  * Ermittlung der freien Plattenkapazitaet
  *
@@ -23,14 +23,18 @@
 #include <sys/param.h>
 #include <sys/mount.h>
 #else
+#if defined( QNX )
+#include <sys/types.h>
+#include <sys/disk.h>
+#else
 /* z.B. SVR3 */
 #include <sys/statfs.h>
+#endif /* QNX */
 #endif /* ultrix */
 #endif /* SVR4 */
 #endif /* WIN32 */
 #endif /* sun / linux / __NeXT__  hpux */
 
-#define MINIMUM( a, b ) ( (a) < (b) ? (a) : (b) )
 
 
 
@@ -40,8 +44,8 @@
 
 int GetDiskParameter( char *path, 
 		      char *volume_name, 
-		      unsigned long *avail_bytes,
-		      unsigned long *total_disk_space
+		      LONGLONG *avail_bytes,
+		      LONGLONG *total_disk_space
 		    )
 {
 
@@ -53,8 +57,13 @@ int GetDiskParameter( char *path,
 #else
 #ifdef ultrix
   struct fs_data statfs_struct;
+#else
+#ifdef QNX
+  long total_blocks, free_blocks;
+  int fd;
 #else
   struct statfs statfs_struct;
+#endif /* QNX */
 #endif /* ultrix */
 #endif /* SVR4 */
 #endif /* WIN32 */
@@ -62,8 +71,8 @@ int GetDiskParameter( char *path,
   char *p;
   char *fname;
   int  result;
-  long bfree;
-  unsigned long disk_space;
+  LONGLONG bfree;
+  LONGLONG this_disk_space;
 
 
 #ifdef WIN32
@@ -72,8 +81,13 @@ int GetDiskParameter( char *path,
 #ifdef __DJGPP__
   if( ( result = statfs( path, &statfs_struct ) ) == 0 )
 #else
+#ifdef QNX
+   fd = open(path, O_RDONLY );
+  if( ( result = disk_space( fd, &free_blocks, &total_blocks ) ) == 0 )
+#else
 
   if( ( result = STATFS( path, &statfs_struct, sizeof( statfs_struct ), 0 ) ) == 0 )
+#endif /* QNX */
 #endif /* __DJGPP__ */
 #endif /* WIN32 */
   {
@@ -122,12 +136,16 @@ int GetDiskParameter( char *path,
 #ifdef __DJGPP__
         fname = "DJGPP";
 #else
+#ifdef QNX
+        fname = "QNX";
+#else
 #if defined( SVR4 ) || defined( OSF1 )
         fname = statfs_struct.f_fstr;
 #else
         fname = statfs_struct.f_fname;
 #endif /* SVR4 */
 #endif /* __DJGPP__ */
+#endif /* QNX */
 #endif /* WIN32 */
 #endif /* sun / hpux / __NeXT__ ultrix */
 #endif /* linux */
@@ -156,35 +174,40 @@ int GetDiskParameter( char *path,
     *avail_bytes = (long) diskspace.bytes_per_sector *
                    (long) diskspace.sectors_per_cluster *
                    (long) diskspace.avail_clusters;
-    disk_space = 900000000L; /* for now.. */
+    this_disk_space = 900000000L; /* for now.. */
 #else
 
 #if (defined( SVR4 ) || defined( OSF1 )) && !defined( __DGUX__ )
     bfree = getuid() ? statfs_struct.f_bavail : statfs_struct.f_bfree;
     if( bfree < 0L ) bfree = 0L;
     *avail_bytes = bfree * statfs_struct.f_frsize;
-    disk_space   = statfs_struct.f_blocks * statfs_struct.f_frsize;
+    this_disk_space   = statfs_struct.f_blocks * statfs_struct.f_frsize;
 #else
 #if defined( _IBMR2 ) || defined( linux ) || defined( sun ) || defined( __NeXT__ )
     bfree = getuid() ? statfs_struct.f_bavail : statfs_struct.f_bfree;
     if( bfree < 0L ) bfree = 0L;
     *avail_bytes = bfree * statfs_struct.f_bsize;
-    disk_space   = statfs_struct.f_blocks * statfs_struct.f_blocks;
+    this_disk_space   = statfs_struct.f_blocks * statfs_struct.f_blocks;
 #else
 #ifdef SVR3
     bfree = statfs_struct.f_bfree;
     if( bfree < 0L ) bfree = 0L;
     *avail_bytes = bfree * BLKSIZ;  /* SYSV */
-    disk_space   = statfs_struct.f_blocks * BLKSIZ;
+    this_disk_space   = statfs_struct.f_blocks * BLKSIZ;
 #else 
 #if defined( ultrix )
     bfree = statfs_struct.fd_req.bfree;
     if( bfree < 0L ) bfree = 0L;
+#else 
+#if defined( QNX )
+    *avail_bytes = free_blocks * 512;
+    this_disk_space = total_blocks * 512;
 #else
     bfree = statfs_struct.f_bfree;
     if( bfree < 0L ) bfree = 0L;
     *avail_bytes = bfree * statfs_struct.f_bsize;
-    disk_space   = statfs_struct.f_blocks * statfs_struct.f_bsize;
+    this_disk_space   = statfs_struct.f_blocks * statfs_struct.f_bsize;
+#endif /* QNX */
 #endif /* ultrix */
 #endif /* SVR3 */
 #endif /* SVR4/!__DGUX__ */
@@ -193,16 +216,19 @@ int GetDiskParameter( char *path,
     
     if( total_disk_space )
     {
-      *total_disk_space = disk_space;
+      *total_disk_space = this_disk_space;
     }
   }
+#ifdef QNX
+  close(fd);
+#endif
   return( result );
 }
 
 
 
 
-int GetAvailBytes(long unsigned int *avail_bytes)
+int GetAvailBytes(LONGLONG *avail_bytes)
 {
   return( GetDiskParameter( statistic.tree->name, 
 			    NULL, 
