@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /home/werner/ytree/RCS/global.c,v 1.9 1997/08/13 12:24:58 werner Rel $
+ * $Header: /usr/local/cvsroot/utils/ytree/global.c,v 1.11 2000/07/13 18:26:06 werner Exp $
  *
  * Externe Groessen
  *
@@ -26,3 +26,6 @@ int       mode;
 int	  user_umask;
 char      message[MESSAGE_LENGTH + 1];
 BOOL	  print_time;
+BOOL	  resize_request;
+BOOL	  bypass_small_window;
+char   number_seperator;
