@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /home/werner/ytree/RCS/history.c,v 1.3 1998/11/11 19:02:48 werner Rel $
+ * $Header: /usr/local/cvsroot/utils/ytree/history.c,v 1.4 2000/05/28 17:48:40 werner Exp $
  *
  * Functionkey F2 History
  *
@@ -252,9 +252,9 @@ char *GetHistory()
   {
     wnoutrefresh( history_window );
     doupdate();
-    ch = getch();
+    ch = Getch();
 
-    if(ch != KEY_RIGHT && ch != KEY_LEFT) {
+    if(ch != -1 && ch != KEY_RIGHT && ch != KEY_LEFT) {
       if(start_x) {
         start_x = 0;
 	PrintHstEntry( disp_begin_pos + cursor_pos,
@@ -265,6 +265,9 @@ char *GetHistory()
 
     switch( ch )
     {
+      case -1:       RetVal = NULL;
+                     break;
+
       case ' ':      break;  /* Quick-Key */
 
       case KEY_RIGHT: start_x++;
@@ -436,7 +439,7 @@ char *GetHistory()
       default :      beep();
 		     break;
     } /* switch */
-  } while(ch != CR && ch != ESC);
+  } while(ch != CR && ch != ESC && ch != -1);
   /* leaveok(stdscr, FALSE); */
   touchwin(stdscr);
   return RetVal;
