@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /home/werner/ytree/RCS/login.c,v 1.15 1998/11/11 19:02:48 werner Rel $
+ * $Header: /usr/local/cvsroot/utils/ytree/login.c,v 1.16 2000/01/01 23:13:53 werner Rel $
  *
  * Dateibaum lesen
  *
@@ -140,6 +140,7 @@ int LoginDisk(char *path)
       case LHA_COMPRESS:    mode = LHA_FILE_MODE; break;
       case ZIP_COMPRESS:    mode = ZIP_FILE_MODE; break;
       case RPM_COMPRESS:    mode = RPM_FILE_MODE; break;
+      case RAR_COMPRESS:    mode = RAR_FILE_MODE; break;
       case TAPE_DIR_NO_COMPRESS:
       case TAPE_DIR_COMPRESS_COMPRESS:
       case TAPE_DIR_FREEZE_COMPRESS:
@@ -240,6 +241,16 @@ int LoginDisk(char *path)
 		      statistic.login_path
 		    );
     }
+    else if( file_method == RAR_COMPRESS )
+    {
+      /* RAR_FILE */
+      /*----------*/
+    
+      (void) sprintf( command_line, "%s '%s'", 
+		      RARLIST, 
+		      statistic.login_path
+		    );
+    }
     else if( file_method == FREEZE_COMPRESS )
     {
       /* melt < TAR_FILE | gtar tvf - */
@@ -505,6 +516,16 @@ int LoginDisk(char *path)
           return( -1 );
 	}
       }
+      else if( mode == RAR_FILE_MODE )
+      {
+	if( ReadTreeFromRAR( statistic.tree, f ) )
+        {
+	  ERROR_MSG( "ReadTreeFromRAR Failed" );
+          (void) fclose( f );
+	  (void) wait( &status );
+          return( -1 );
+	}
+      }
       else
       {
         if( ReadTreeFromTAR( statistic.tree, f ) )
