@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /home/werner/ytree/RCS/main.c,v 1.10 1998/07/26 21:21:13 werner Rel $
+ * $Header: /usr/local/cvsroot/utils/ytree/main.c,v 1.12 2000/07/13 18:26:06 werner Exp $
  *
  * Hauptmodul
  *
@@ -49,15 +49,15 @@ int main(int argc, char **argv)
   if( LoginDisk( path ) == -1 )
   {
     endwin();
+#ifdef XCURSES
+    XCursesExit();
+#endif
     exit( 1 );
   }
 
   while( 1 )
   {
-    if( HandleDirWindow( statistic.tree, 
-			 statistic.disp_begin_pos,
-			 statistic.cursor_pos
-		       ) == 'q' ) Quit();
+    if( HandleDirWindow(statistic.tree) == 'q' ) Quit();
   }
 }
 
