@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /home/werner/ytree/RCS/match.c,v 1.11 1998/11/11 19:02:48 werner Rel $
+ * $Header: /usr/local/cvsroot/utils/ytree/match.c,v 1.12 2000/01/01 23:13:53 werner Rel $
  *
  * Behandlung reg. Ausdruecke fuer Dateinamen
  *
@@ -91,7 +91,7 @@ int SetMatchSpec(char *new_spec)
 
 #if defined ( HAS_REGEX )
 
-  if( ( result = re_comp( buffer ) ) )
+  if( ( result = (char *) re_comp( buffer ) ) )
   {
     free( buffer );
     return( 1 );
