@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /home/werner/ytree/RCS/sort.c,v 1.9 1998/11/11 19:02:48 werner Rel $
+ * $Header: /usr/local/cvsroot/utils/ytree/sort.c,v 1.10 2000/05/28 17:48:40 werner Exp $
  *
  * Umschalten des Sortierkriteriums
  *
@@ -29,8 +29,12 @@ void GetKindOfSort(void)
   doupdate();
   do 
   {
-        if ((c = toupper(getch())) == ESC) 
-           return;
+        c = Getch();
+	if(c == -1 || c == ESC)
+	  return;
+
+        c = toupper(c);
+
 	if (c == 'Q')
 	  return;
 
