@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /home/werner/ytree/RCS/util.c,v 1.16 1998/11/11 19:02:48 werner Rel $
+ * $Header: /usr/local/cvsroot/utils/ytree/util.c,v 1.23 2000/07/13 18:26:06 werner Exp $
  *
  * Diverse Hilfsfunktionen
  *
@@ -129,12 +129,14 @@ int GetDirEntry(DirEntry *tree,
     return( -3 );
   }
 
-#if 0
-  (void) getcwd( dest_path, sizeof( dest_path ) - 2 );
-  (void) strcpy( to_path, dest_path );
-#endif
+  if( *dir_path != FILE_SEPARATOR_CHAR ) {
+    (void) getcwd( dest_path, sizeof( dest_path ) - 2 );
+    (void) strcpy( to_path, dest_path );
+  } else {
+    strcpy(dest_path, dir_path);
+  }
+
 
-  strcpy(dest_path, dir_path);
   if( chdir( current_path ) )
   {
     ERROR_MSG( "Chdir failed; Can't resume" );
@@ -142,8 +144,9 @@ int GetDirEntry(DirEntry *tree,
   }
 
   n = strlen( tree->name );
-  if( !strncmp( tree->name, dest_path, n ) &&
-      ( dest_path[n] == FILE_SEPARATOR_CHAR || dest_path[n] == '\0' ) )
+  if( !strcmp(tree->name, FILE_SEPARATOR_STRING) || 
+      (!strncmp( tree->name, dest_path, n )     &&
+        ( dest_path[n] == FILE_SEPARATOR_CHAR || dest_path[n] == '\0' ) ) )
   {
     /* Pfad befindet sich im (Sub)-Tree */
     /*----------------------------------*/
@@ -286,6 +289,7 @@ int InputString(char *s, int y, int x, int length, char *term)
   /* Feld gefuellt ausgeben */
   /*------------------------*/
   print_time = FALSE;
+  curs_set(1);
   MvAddStr( y, x, s );
   leaveok(stdscr, FALSE);
   
@@ -302,7 +306,7 @@ int InputString(char *s, int y, int x, int length, char *term)
     wnoutrefresh( stdscr );
     doupdate();
 
-    c1 = getch();
+    c1 = Getch();
 
     if( c1 >= ' ' && c1 < 0xff && c1 != 127) 
     {
@@ -362,6 +366,8 @@ int InputString(char *s, int y, int x, int length, char *term)
 
       switch( c1 )
       {
+        case -1             : break;
+
         case KEY_LEFT       : if( p > 0 )
                                 p--;
                               else
@@ -448,7 +454,7 @@ int InputString(char *s, int y, int x, int length, char *term)
       }
     }
     key_pressed = TRUE;
-  } while( !strrchr( term, c1 ) );
+  } while( c1 != -1 && !strrchr( term, c1 ) );
 
   p = strlen( s );
 
@@ -459,6 +465,7 @@ int InputString(char *s, int y, int x, int length, char *term)
 
   move( y, x );
   leaveok( stdscr, TRUE);
+  curs_set(0);
   print_time = TRUE;
   InsHistory( s );
   return( c1 );
@@ -474,20 +481,24 @@ int InputChoise(char *msg, char *term)
 
   ClearHelp();
 
+  curs_set(1);
   leaveok(stdscr, FALSE);
   mvprintw( LINES - 2, 1, msg );
   wnoutrefresh( stdscr );
   doupdate();
   do
   {
-    c = getch();
-    if( islower( c ) ) c = toupper( c );
-  } while( !strchr( term, c ) );
+    c = Getch();
+    if(c >= 0)
+      if( islower( c ) ) c = toupper( c );
+  } while( c != -1 && !strchr( term, c ) );
   
-  echochar( c );
+  if(c >= 0)
+    echochar( c );
 
   move( LINES - 2, 1 ); clrtoeol();
   leaveok(stdscr, TRUE);
+  curs_set(0);
 
   return( c );
 }
@@ -555,6 +566,11 @@ void PrintSpecialString(WINDOW *win, int y, int x, char *str, int color)
 {
   int ch;
 
+  if(x < 0 || y < 0) {
+     /* screen too small */
+    return;
+  }
+
   for( ; *str; str++ )
   {
     switch( *str )
@@ -584,6 +600,11 @@ void PrintOptions(WINDOW *win, int y, int x, char *str)
   int ch;
   int color, hi_color, lo_color;
 
+  if(x < 0 || y < 0) {
+     /* screen too small */
+    return;
+  }
+
 #ifdef COLOR_SUPPORT
      lo_color = MENU_COLOR;
      hi_color = HIMENUS_COLOR;
@@ -639,6 +660,11 @@ void PrintMenuOptions(WINDOW *win,int y, int x, char *str, int ncolor, int hcolo
   int ch;
   int color, hi_color, lo_color;
 
+  if(x < 0 || y < 0) {
+     /* screen too small */
+    return;
+  }
+
 #ifdef COLOR_SUPPORT
      lo_color = MENU_COLOR;
      hi_color = HIMENUS_COLOR;
@@ -766,11 +792,15 @@ void Fnsplit(char *path, char *dir, char *name)
 
 void HitReturnToContinue(void)
 {
+#ifndef XCURSES
+  curs_set(1);
   vidattr( A_REVERSE );
   putp( "[Hit return to continue]" );
   vidattr( 0 );
   (void) fflush( stdout );
-  (void) getch();
+  (void) Getch();
+#endif /* XCURSES */
+  curs_set(0);
   doupdate();
 }
 
@@ -1034,28 +1064,28 @@ void GetMaxYX(WINDOW *win, int *height, int *width)
 {
   if( win == dir_window )
   { 
-    *height = DIR_WINDOW_HEIGHT;
-    *width  = DIR_WINDOW_WIDTH;
+    *height = MAXIMUM(DIR_WINDOW_HEIGHT, 1);
+    *width  = MAXIMUM(DIR_WINDOW_WIDTH, 1);
   }
   else if( win == small_file_window )
   {
-    *height = FILE_WINDOW_1_HEIGHT;
-    *width  = FILE_WINDOW_1_WIDTH;
+    *height = MAXIMUM(FILE_WINDOW_1_HEIGHT, 1);
+    *width  = MAXIMUM(FILE_WINDOW_1_WIDTH, 1);
   }
   else if( win == big_file_window )
   {
-    *height = FILE_WINDOW_2_HEIGHT;
-    *width  = FILE_WINDOW_2_WIDTH;
+    *height = MAXIMUM(FILE_WINDOW_2_HEIGHT, 1);
+    *width  = MAXIMUM(FILE_WINDOW_2_WIDTH, 1);
   }
   else if( win == f2_window )
   {
-    *height = F2_WINDOW_HEIGHT - 1; /* fake for separator line */
-    *width  = F2_WINDOW_WIDTH;
+    *height = MAXIMUM(F2_WINDOW_HEIGHT - 1, 1); /* fake for separator line */
+    *width  = MAXIMUM(F2_WINDOW_WIDTH, 1);
   }
   else if( win == history_window )
   {
-    *height = HISTORY_WINDOW_HEIGHT;
-    *width  = HISTORY_WINDOW_WIDTH;
+    *height = MAXIMUM(HISTORY_WINDOW_HEIGHT, 1);
+    *width  = MAXIMUM(HISTORY_WINDOW_WIDTH, 1);
   }
   else
   {
@@ -1123,3 +1153,204 @@ void StrCp(char *dest, const char *src)
    *dest = '\0';
 }
 
+
+
+
+int BuildUserFileEntry(FileEntry *fe_ptr, 
+			int max_filename_len, int max_linkname_len, 
+			char *template, int linelen, char *line)
+{
+  char attributes[11];
+  char modify_time[13];
+  char change_time[13];
+  char access_time[13];
+  char format1[60];
+  char format2[60];
+  int  i, n;
+  char owner[OWNER_NAME_MAX + 1];
+  char group[GROUP_NAME_MAX + 1];
+  char *owner_name_ptr;
+  char *group_name_ptr;
+  char *sym_link_name = NULL;
+  char *sptr, *dptr;
+  char tag;
+  char buffer[4096]; /* enough??? */
+
+
+  if( fe_ptr && S_ISLNK( fe_ptr->stat_struct.st_mode ) )
+    sym_link_name = &fe_ptr->name[strlen(fe_ptr->name)+1];
+  else
+    sym_link_name = "";
+
+
+  tag = (fe_ptr->tagged) ? TAGGED_SYMBOL : ' ';
+  (void) GetAttributes( fe_ptr->stat_struct.st_mode, attributes);
+		        
+  (void) CTime( fe_ptr->stat_struct.st_mtime, modify_time );
+  (void) CTime( fe_ptr->stat_struct.st_ctime, change_time );
+  (void) CTime( fe_ptr->stat_struct.st_atime, access_time );
+  
+  owner_name_ptr = GetPasswdName(fe_ptr->stat_struct.st_uid);
+  group_name_ptr = GetGroupName(fe_ptr->stat_struct.st_gid);
+
+  if( owner_name_ptr == NULL )
+  {
+    (void) sprintf( owner, "%d", (int) fe_ptr->stat_struct.st_uid );
+    owner_name_ptr = owner;
+  }
+  if( group_name_ptr == NULL )
+  {
+    (void) sprintf( group, "%d", (int) fe_ptr->stat_struct.st_gid );
+    group_name_ptr = group;
+  }
+
+
+  sprintf(format1, "%%-%ds", max_filename_len);
+  sprintf(format2, "%%-%ds", max_linkname_len);
+
+  for(sptr=template, dptr=buffer, i=0; *sptr; ) {
+
+    if(*sptr == '%') {
+      sptr++;
+      if(!strncmp(sptr, TAGSYMBOL_VIEWNAME, 3)) {
+        *dptr = tag; n=1;
+      } else if(!strncmp(sptr, FILENAME_VIEWNAME, 3)) {
+        n = sprintf(dptr, format1, fe_ptr->name);
+      } else if(!strncmp(sptr, ATTRIBUTE_VIEWNAME, 3)) {
+        n = sprintf(dptr, "%10s", attributes);
+      } else if(!strncmp(sptr, LINKCOUNT_VIEWNAME, 3)) {
+        n = sprintf(dptr, "%3d", fe_ptr->stat_struct.st_nlink);
+      } else if(!strncmp(sptr, FILESIZE_VIEWNAME, 3)) {
+#ifdef HAS_LONGLONG
+        n = sprintf(dptr, "%7lld", (LONGLONG) fe_ptr->stat_struct.st_size);
+#else
+        n = sprintf(dptr, "%7d", fe_ptr->stat_struct.st_size);
+#endif
+      } else if(!strncmp(sptr, MODTIME_VIEWNAME, 3)) {
+        n = sprintf(dptr, "%12s", modify_time);
+      } else if(!strncmp(sptr, SYMLINK_VIEWNAME, 3)) {
+        n = sprintf(dptr, format2, sym_link_name);
+      } else if(!strncmp(sptr, UID_VIEWNAME, 3)) {
+        n = sprintf(dptr, "%-8s", owner_name_ptr);
+      } else if(!strncmp(sptr, GID_VIEWNAME, 3)) {
+        n = sprintf(dptr, "%-8s", group_name_ptr);
+      } else if(!strncmp(sptr, INODE_VIEWNAME, 3)) {
+        n = sprintf(dptr, "%7ld", fe_ptr->stat_struct.st_ino);
+      } else if(!strncmp(sptr, ACCTIME_VIEWNAME, 3)) {
+        n = sprintf(dptr, "%12s", access_time);
+      } else if(!strncmp(sptr, CHGTIME_VIEWNAME, 3)) {
+        n = sprintf(dptr, "%12s", change_time);
+      } else {
+	n = -1;
+      }
+      if(n == -1) {
+        *dptr++ = '%';
+	} else {
+        dptr += n;
+        if(*sptr) sptr++;
+        if(*sptr) sptr++;
+        if(*sptr) sptr++;
+      }
+    } else {
+      *dptr++ = *sptr++;
+    }
+  }
+
+  strncpy(line, buffer, linelen);
+  line[linelen - 1] = '\0';
+  return(0);
+}
+  
+
+
+int GetUserFileEntryLength( int max_filename_len, int max_linkname_len, char *template)
+{
+  int  len, n;
+  char *sptr;
+
+
+  for(len=0, sptr=template; *sptr; ) {
+
+    if(*sptr == '%') {
+      sptr++;
+      if(!strncmp(sptr, TAGSYMBOL_VIEWNAME, 3)) {
+        n=1;
+      } else if(!strncmp(sptr, FILENAME_VIEWNAME, 3)) {
+        n = max_filename_len;
+      } else if(!strncmp(sptr, ATTRIBUTE_VIEWNAME, 3)) {
+        n = 10;
+      } else if(!strncmp(sptr, LINKCOUNT_VIEWNAME, 3)) {
+        n = 3;
+      } else if(!strncmp(sptr, FILESIZE_VIEWNAME, 3)) {
+        n = 7;
+      } else if(!strncmp(sptr, MODTIME_VIEWNAME, 3)) {
+        n = 12;
+      } else if(!strncmp(sptr, SYMLINK_VIEWNAME, 3)) {
+        n = max_linkname_len;
+      } else if(!strncmp(sptr, UID_VIEWNAME, 3)) {
+        n = 8;
+      } else if(!strncmp(sptr, GID_VIEWNAME, 3)) {
+        n = 8;
+      } else if(!strncmp(sptr, INODE_VIEWNAME, 3)) {
+        n = 7;
+      } else if(!strncmp(sptr, ACCTIME_VIEWNAME, 3)) {
+        n = 12;
+      } else if(!strncmp(sptr, CHGTIME_VIEWNAME, 3)) {
+        n = 12;
+      } else {
+	n = -1;
+      }
+      if(n == -1) {
+        len++;
+	sptr++;
+	} else {
+        len += n;
+        if(*sptr) sptr++;
+        if(*sptr) sptr++;
+        if(*sptr) sptr++;
+      }
+    } else {
+      sptr++;
+      len++;
+    }
+  }
+  return(len);
+}
+  
+
+LONGLONG AtoLL(char *cptr)
+{
+  LONGLONG ll;
+
+#ifdef HAS_LONGLONG
+
+  sscanf(cptr, "%lld", &ll);
+
+#else
+#ifdef __QNX__
+  sscanf(cptr, "%ld", &ll);
+#else
+  sscanf(ll, "%ld", cptr);
+#endif
+#endif
+
+  return(ll);
+}
+
+
+int Getch()
+{
+  int c;
+
+  c = getch();
+
+#ifdef KEY_RESIZE
+  if(c == KEY_RESIZE) {
+    resize_request = TRUE;
+    c = -1;
+  }
+#endif
+
+  return(c);
+}
+
