@@ -1,28 +1,42 @@
 /***************************************************************************
  *
- * $Header: /home/werner/ytree/RCS/ytree.h,v 1.19 1998/11/11 19:02:48 werner Rel $
+ * $Header: /usr/local/cvsroot/utils/ytree/ytree.h,v 1.26 2000/07/15 21:59:27 werner Exp $
  *
  * Header-Datei fuer YTREE
  *
  ***************************************************************************/
 
 
+#define _LARGEFILE64_SOURCE 1
+#define _FILE_OFFSET_BITS 64                                                                   
+
 
 #include <stdio.h>
 #include <ctype.h>
 #include <math.h>
 
+#ifdef XCURSES
+#include <xcurses.h>
+#define HAVE_CURSES 1
+#endif
+
 #ifdef _IBMR2
 #define NLS
 #endif /* _IBMR2 */
 
 #ifdef ultrix
+#ifndef HAVE_CURSES
 #include <cursesX.h>
+#endif
 #else
 #ifdef __FreeBSD__
+#ifndef HAVE_CURSES
 #include <ncurses.h>
+#endif
 #else
+#ifndef HAVE_CURSES
 #include <curses.h>
+#endif
 #endif /* __FreeBSD__ */
 #endif /* ultrix */
 
@@ -52,7 +66,9 @@ extern char *getcwd();
 #include <errno.h>
 #include <fcntl.h>
 #include <grp.h>
+#ifndef __QNX__
 #include <memory.h>
+#endif
 #include <pwd.h>
 #include <setjmp.h>
 #include <signal.h>
@@ -64,33 +80,68 @@ extern char *getcwd();
 
 #ifdef __FreeBSD__
 #define STATFS(a, b, c, d )     statfs( a, b )
+#define LONGLONG		long long
+#define HAS_LONGLONG		1
 #else
 #ifdef OSF1
 #define STATFS(a, b, c, d )     statvfs( a, b )
 #define echochar( ch )          { addch( ch ); refresh(); }
+#define LONGLONG		unsigned long
 #else
 #ifdef SVR4
 #define STATFS(a, b, c, d )     statvfs( a, b )
+#define LONGLONG		unsigned long
+#else
+#ifdef SVR3
+#define LONGLONG		unsigned long
+#define STATFS(a, b, c, d )     statfs( a, b, c, d )
+#define LONGLONG		unsigned long
+#else
+#ifdef _IBMR2
+#define STATFS(a, b, c, d )     statfs( a, b )
+#define LONGLONG		long long
+#define HAS_LONGLONG		1
 #else
 #ifdef linux
 #define STATFS(a, b, c, d )     statfs( a, b )
+#define LONGLONG		long long
+#define HAS_LONGLONG		1
 #else
 #ifdef hpux
 #define STATFS(a, b, c, d )     statfs( a, b )
 #define echochar( ch )          { addch( ch ); refresh(); }
+#define LONGLONG		long long
+#define HAS_LONGLONG		1
 #else
 #ifdef ultrix
 #define STATFS(a, b, c, d )	statfs( a, b )
 #define echochar( ch )          { addch( ch ); refresh(); }
+#define LONGLONG		unsigned long
+#else
+#ifdef __QNX__
+#define STATFS(a, b, c, d )	statfs( a, b )
+#define LONGLONG		unsigned long
 #else
 #define STATFS(a, b, c, d )     statfs( a, b, c, d )
+#define LONGLONG		long long
+#define HAS_LONGLONG		1
+#endif /* __QNX__ */
 #endif /* ultrix */
 #endif /* hpux */
 #endif /* linux */
+#endif /* _IBMR2 */
 #endif /* SVR4 */
+#endif /* SVR3 */
 #endif /* OSF1 */
 #endif /* __FreeBSD__ */
 
+
+/* Some handy macros... */
+
+#define MINIMUM( a, b ) ( ( (a) < (b) ) ? (a) : (b) )
+#define MAXIMUM( a, b ) ( ( (a) > (b) ) ? (a) : (b) )
+
+
 #ifdef WIN32
 
 #define  S_IREAD         S_IRUSR
@@ -230,7 +281,7 @@ extern char *getcwd();
 #define KEY_END   KEY_EOL
 #endif
 
-#ifdef _IBMR2
+#if defined(_IBMR2) && !defined(_AIX41)
 #define echochar( c )  { addch( c ); refresh(); }
 #define wgetch( w )    AixWgetch( w )
 #endif
@@ -369,28 +420,34 @@ extern char *getcwd();
 /* Auswahl der benutzten UNIX-Kommandos */
 /*--------------------------------------*/
 
-#define CAT         GetProfileValue( "CAT" )
-#define HEXDUMP     GetProfileValue( "HEXDUMP" )
-#define EDITOR      GetProfileValue( "EDITOR" )
-#define PAGER       GetProfileValue( "PAGER" )
-#define MELT        GetProfileValue( "MELT" )
-#define UNCOMPRESS  GetProfileValue( "UNCOMPRESS" )
-#define GNUUNZIP    GetProfileValue( "GNUUNZIP" )
-#define BUNZIP      GetProfileValue( "BUNZIP" )
-#define MANROFF     GetProfileValue( "MANROFF" )
-#define TARLIST     GetProfileValue( "TARLIST" )
-#define TAREXPAND   GetProfileValue( "TAREXPAND" )
-#define RPMLIST     GetProfileValue( "RPMLIST" )
-#define RPMEXPAND   GetProfileValue( "RPMEXPAND" )
-#define ZOOLIST     GetProfileValue( "ZOOLIST" )
-#define ZOOEXPAND   GetProfileValue( "ZOOEXPAND" )
-#define ZIPLIST     GetProfileValue( "ZIPLIST" )
-#define ZIPEXPAND   GetProfileValue( "ZIPEXPAND" )
-#define LHALIST     GetProfileValue( "LHALIST" )
-#define LHAEXPAND   GetProfileValue( "LHAEXPAND" )
-#define ARCLIST     GetProfileValue( "ARCLIST" )
-#define ARCEXPAND   GetProfileValue( "ARCEXPAND" )
-#define TREEDEPTH   GetProfileValue( "TREEDEPTH" )
+#define CAT           GetProfileValue( "CAT" )
+#define HEXDUMP       GetProfileValue( "HEXDUMP" )
+#define EDITOR        GetProfileValue( "EDITOR" )
+#define PAGER         GetProfileValue( "PAGER" )
+#define MELT          GetProfileValue( "MELT" )
+#define UNCOMPRESS    GetProfileValue( "UNCOMPRESS" )
+#define GNUUNZIP      GetProfileValue( "GNUUNZIP" )
+#define BUNZIP        GetProfileValue( "BUNZIP" )
+#define MANROFF       GetProfileValue( "MANROFF" )
+#define TARLIST       GetProfileValue( "TARLIST" )
+#define TAREXPAND     GetProfileValue( "TAREXPAND" )
+#define RPMLIST       GetProfileValue( "RPMLIST" )
+#define RPMEXPAND     GetProfileValue( "RPMEXPAND" )
+#define ZOOLIST       GetProfileValue( "ZOOLIST" )
+#define ZOOEXPAND     GetProfileValue( "ZOOEXPAND" )
+#define ZIPLIST       GetProfileValue( "ZIPLIST" )
+#define ZIPEXPAND     GetProfileValue( "ZIPEXPAND" )
+#define LHALIST       GetProfileValue( "LHALIST" )
+#define LHAEXPAND     GetProfileValue( "LHAEXPAND" )
+#define ARCLIST       GetProfileValue( "ARCLIST" )
+#define ARCEXPAND     GetProfileValue( "ARCEXPAND" )
+#define TREEDEPTH     GetProfileValue( "TREEDEPTH" )
+#define USERVIEW      GetProfileValue( "USERVIEW" )
+#define RARLIST       GetProfileValue( "RARLIST" )
+#define RAREXPAND     GetProfileValue( "RAREXPAND" )
+#define FILEMODE      GetProfileValue( "FILEMODE" )
+#define NUMBERSEP     GetProfileValue( "NUMBERSEP" )
+#define NOSMALLWINDOW GetProfileValue( "NOSMALLWINDOW" )
  
 
 #define DEFAULT_TREE       "."
@@ -402,7 +459,7 @@ extern char *getcwd();
 #define NOTICE( msg )      Notice( msg ) 
 
 #define TAGGED_SYMBOL '*'
-#define MAX_MODES      9
+#define MAX_MODES      10
 #define DISK_MODE      0
 #define LL_FILE_MODE   1
 #define TAR_FILE_MODE  2
@@ -411,7 +468,8 @@ extern char *getcwd();
 #define LHA_FILE_MODE  5
 #define ARC_FILE_MODE  6
 #define RPM_FILE_MODE  7
-#define TAPE_MODE      8 
+#define RAR_FILE_MODE  8
+#define TAPE_MODE      9 
 
 #define NO_COMPRESS                 0
 #define FREEZE_COMPRESS             1
@@ -431,6 +489,7 @@ extern char *getcwd();
 #define TAPE_DIR_COMPRESS_COMPRESS  15
 #define TAPE_DIR_GZIP_COMPRESS      16
 #define TAPE_DIR_BZIP_COMPRESS	    17
+#define RAR_COMPRESS                18
 
 #define FILE_EXTENSIONS  {                                           \
 			   { ".TAP", TAPE_DIR_NO_COMPRESS },         \
@@ -469,12 +528,16 @@ extern char *getcwd();
 			   { ".LZH", LHA_COMPRESS },                 \
 			   { ".arc", ARC_COMPRESS },                 \
 			   { ".ARC", ARC_COMPRESS },                 \
+			   { ".rar", RAR_COMPRESS },                 \
+			   { ".RAR", RAR_COMPRESS },                 \
 			   { ".jar", ZIP_COMPRESS },                 \
 			   { ".zip", ZIP_COMPRESS },                 \
 			   { ".ZIP", ZIP_COMPRESS },                 \
 			   { ".JAR", ZIP_COMPRESS },                 \
 			   { ".rpm", RPM_COMPRESS },                 \
-			   { ".RPM", RPM_COMPRESS }                  \
+			   { ".RPM", RPM_COMPRESS },                 \
+			   { ".spm", RPM_COMPRESS },                 \
+			   { ".SPM", RPM_COMPRESS }                  \
 			 }
 
 
@@ -494,6 +557,19 @@ extern char *getcwd();
 
 #define DEFAULT_FILE_SPEC "*"
 
+#define TAGSYMBOL_VIEWNAME	"tag"
+#define FILENAME_VIEWNAME	"fnm"
+#define ATTRIBUTE_VIEWNAME	"atr"
+#define LINKCOUNT_VIEWNAME	"lct"
+#define FILESIZE_VIEWNAME	"fsz"
+#define MODTIME_VIEWNAME	"mot"
+#define SYMLINK_VIEWNAME	"lnm"
+#define UID_VIEWNAME		"uid"
+#define GID_VIEWNAME		"gid"
+#define INODE_VIEWNAME		"ino"
+#define ACCTIME_VIEWNAME	"act"
+#define CHGTIME_VIEWNAME	"sct"
+
 
 #define BLKSIZ             512  /* Blockgroesse fuer SVR3 */
 
@@ -514,10 +590,7 @@ extern char *getcwd();
 #define SGI_CR     0x157    /* Irix 3.2.2 special ? */
 #endif
 
-#define CR         13
-
-#define MIN_WIDTH  41
-#define MIN_HEIGHT 24
+#define CR                     13
 
 #define DIR_WINDOW_X         1  
 #define DIR_WINDOW_Y         2 
@@ -549,27 +622,30 @@ extern char *getcwd();
 #define HISTORY_WINDOW_WIDTH   (COLS - 23)
 #define HISTORY_WINDOW_HEIGHT  (LINES - 5)
 
-#define TIME_WINDOW_X        COLS - 18
+#define TIME_WINDOW_X        ((COLS > 18) ? (COLS - 18) : 1)
 #define TIME_WINDOW_Y        1
-#define TIME_WINDOW_WIDTH    15
+#define TIME_WINDOW_WIDTH    ((COLS > 15) ? 15 : COLS)
 #define TIME_WINDOW_HEIGHT   1
 
-#define PATH_LENGTH          1024
-#define FILE_SPEC_LENGTH     (12 + 1)
-#define DISK_NAME_LENGTH     (12 + 1)
-#define LL_LINE_LENGTH       512
-#define TAR_LINE_LENGTH      512
-#define RPM_LINE_LENGTH      512
-#define ZOO_LINE_LENGTH      512
-#define ZIP_LINE_LENGTH      512
-#define LHA_LINE_LENGTH      512
-#define ARC_LINE_LENGTH      512
-#define MESSAGE_LENGTH       (PATH_LENGTH + 80 + 1)
-#define COMMAND_LINE_LENGTH  4096
-#define MODE_1               0
-#define MODE_2               1
-#define MODE_3               2
-#define MODE_4               3
+
+#define PATH_LENGTH            1024
+#define FILE_SPEC_LENGTH       (12 + 1)
+#define DISK_NAME_LENGTH       (12 + 1)
+#define LL_LINE_LENGTH         512
+#define TAR_LINE_LENGTH        512
+#define RPM_LINE_LENGTH        512
+#define ZOO_LINE_LENGTH        512
+#define ZIP_LINE_LENGTH        512
+#define LHA_LINE_LENGTH        512
+#define ARC_LINE_LENGTH        512
+#define RAR_LINE_LENGTH        512
+#define MESSAGE_LENGTH         (PATH_LENGTH + 80 + 1)
+#define COMMAND_LINE_LENGTH    4096
+#define MODE_1                 0
+#define MODE_2                 1
+#define MODE_3                 2
+#define MODE_4                 3
+#define MODE_5                 4
 
 
 #ifdef __NeXT__
@@ -616,9 +692,9 @@ typedef struct _dir_entry
   struct _dir_entry  *prev;
   struct _dir_entry  *sub_tree;
   struct _dir_entry  *up_tree;
-  unsigned long      total_bytes;
-  unsigned long      matching_bytes;
-  unsigned long      tagged_bytes;
+  LONGLONG           total_bytes;
+  LONGLONG           matching_bytes;
+  LONGLONG           tagged_bytes;
   unsigned int       total_files;
   unsigned int       matching_files;
   unsigned int       tagged_files;
@@ -652,14 +728,14 @@ typedef struct
 typedef struct 
 {
   DirEntry      *tree;
-  unsigned long disk_space;
-  unsigned long disk_capacity;
-  unsigned long disk_total_files;
-  unsigned long disk_total_bytes;
-  unsigned long disk_matching_files;
-  unsigned long disk_matching_bytes;
-  unsigned long disk_tagged_files;
-  unsigned long disk_tagged_bytes;
+  LONGLONG	disk_space;
+  LONGLONG	disk_capacity;
+  LONGLONG	disk_total_files;
+  LONGLONG	disk_total_bytes;
+  LONGLONG	disk_matching_files;
+  LONGLONG	disk_matching_bytes;
+  LONGLONG	disk_tagged_files;
+  LONGLONG 	disk_tagged_bytes;
   unsigned int  disk_total_directories;
   int           disp_begin_pos;
   int           cursor_pos;
@@ -759,6 +835,10 @@ extern int       mode;
 extern int       user_umask;
 extern char      message[];
 extern BOOL	 print_time;
+extern BOOL      resize_request;
+extern char      number_seperator;
+extern BOOL      bypass_small_window;
+
 
 #if defined(ultrix)
 extern char *getenv(char *);
@@ -773,6 +853,7 @@ extern void DisplayDirStatistic(DirEntry *dir_entry);
 extern void DisplayDirParameter(DirEntry *dir_entry);
 extern void DisplayDirTagged(DirEntry *dir_entry);
 extern void DisplayDiskTagged(void);
+extern void DisplayDiskName(void);
 extern void DisplayFileParameter(FileEntry *file_entry);
 extern void DisplayGlobalFileParameter(FileEntry *file_entry);
 extern int  ReadTree(DirEntry *dir_entry, char *path, int depth);
@@ -783,16 +864,14 @@ extern int  ReadTreeFromZOO(DirEntry *dir_entry, FILE *f);
 extern int  ReadTreeFromZIP(DirEntry *dir_entry, FILE *f);
 extern int  ReadTreeFromLHA(DirEntry *dir_entry, FILE *f);
 extern int  ReadTreeFromARC(DirEntry *dir_entry, FILE *f);
+extern int  ReadTreeFromRAR(DirEntry *dir_entry, FILE *f);
 extern int  GetDiskParameter(char *path, 
 			     char *volume_name, 
-			     long unsigned int *avail_bytes,
-			     long unsigned int *capacity
+			     LONGLONG *avail_bytes,
+			     LONGLONG *capacity
 			    );
-extern int  HandleDirWindow(DirEntry *start_dir_entry,
-			    int disp_begin_pos,
-			    int cursor_pos
-			   );
-extern void DisplayFileWindow(DirEntry *dir_entry, int start_file_no, int hilight_no);
+extern int  HandleDirWindow(DirEntry *start_dir_entry);
+extern void DisplayFileWindow(DirEntry *dir_entry);
 extern int  Init(void);
 extern char *GetPath(DirEntry *dir_entry, char *buffer);
 extern BOOL Match(char *file_name);
@@ -804,11 +883,7 @@ extern void Warning(char *msg);
 extern void Notice(char *msg);
 extern void UnmapNoticeWindow(void);
 extern void SetFileMode(int new_file_mode);
-extern int  HandleFileWindow(DirEntry *dir_entry, 
-			     int start_file,
-			     int cursor_pos,
-			     BOOL global_flag,
-			     BOOL big_window);
+extern int  HandleFileWindow(DirEntry *dir_entry);
 extern char *GetAttributes(unsigned short modus, char *buffer);
 extern void SwitchToSmallFileWindow(void);
 extern void SwitchToBigFileWindow(void);
@@ -829,7 +904,7 @@ extern int  Edit(char *file_path);
 extern void DisplayAvailBytes(void);
 extern void DisplayFileSpec(void);
 extern void Quit(void);
-extern void ReadFileSpec(void);
+extern int  ReadFileSpec(void);
 extern int  InputString(char *s, int y, int x, int length, char *term);
 extern void RotateFileMode(void);
 extern int  Execute(DirEntry *dir_entry, FileEntry *file_entry);
@@ -864,7 +939,7 @@ extern int  ChangeDirGroup(DirEntry *de_ptr);
 extern void DisplayDirHelp(void);
 extern void DisplayFileHelp(void);
 extern void ClearHelp(void);
-extern int  GetAvailBytes(long unsigned int *avail_bytes);
+extern int  GetAvailBytes(LONGLONG *avail_bytes);
 extern int  DeleteDirectory(DirEntry *dir_entry);
 extern int  ExecuteCommand(FileEntry *fe_ptr, WalkingPackage *walking_package);
 extern int  GetCommandLine(char *command_line);
@@ -934,3 +1009,13 @@ extern char *GetExtViewer(char *filename);
 extern void InitClock(void);
 extern char *GetExtension(char *filename);
 extern void StrCp(char *dest, const char *src);
+extern int  BuildUserFileEntry(FileEntry *fe_ptr, 
+            int max_filename_len, int max_linkname_len, 
+            char *template, int linelen, char *line);
+extern int  GetUserFileEntryLength(int max_filename_len, 
+				   int max_linkname_len, char *template);
+extern LONGLONG AtoLL(char* cptr);
+extern void DisplayTree(WINDOW *win, int start_entry_no, int hilight_no);
+extern void ReCreateWindows(void);
+extern int  Getch(void);
+
